%--------------------------------anls_test--------------------------------%
% This file tests the ANLS algorithm.
%
% External functions:
%
% - kr.m              : Khatri-Rao product;
% - anls.m            : ANLS algorithm;
% - amb_correct.m     : Matching Permutation for two CP models.
%
% List of updates                 -     23/07/2015  -     R. C. Farias 
%                                       Creation of the file
%-------------------------------------------------------------------------%

%--------------Cleaning the workspace and loading functions---------------%
clc;
clear     all;
close     all;
%-------------------------------------------------------------------------%

%-----------------------------Model parameters----------------------------%
% 3-way block dimensions
dim       =     [100 100 100];
% Number of components
R         =     5;
% Standard deviations of the measurements noise
sigma_n   =     0.000001;
% Number of iterations
iter      =     1000;
%-------------------------------------------------------------------------%

%------------------------------Tensor generation--------------------------%
% Random factors generation
A         =     abs(randn(dim(1),R));
B         =     abs(randn(dim(2),R));
C         =     abs(randn(dim(3),R));
% Normalization
A         =     A.*repmat(1./sum(A),dim(1),1);
B         =     B.*repmat(1./sum(B),dim(2),1);
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%
%Noisy measurements
% Noisy tensor
Y           =     reshape(A*transpose(kr(C,B)),dim)+...
                  sigma_n*randn(dim);
            
%-----------------------------Initialization------------------------------%
A_0         =     abs(randn(size(A)));
B_0         =     abs(randn(size(B)));
A_0         =     A_0.*repmat(1./sum(A_0),dim(1),1);
B_0         =     B_0.*repmat(1./sum(B_0),dim(2),1);
C_0         =     abs(randn(size(C)));
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%

%--------------------------ALS updated version----------------------------%
tic
% ALS
[A_anls,B_anls,C_anls,err_anls_vec]     =     anls(Y,iter,A_0,B_0,C_0);
toc
% Resulting factors
% Correcting permutation and sign ambiguities
[A_anls,B_anls,C_anls]=     amb_correct(A_anls,B_anls,C_anls,A,B,C);
% Column of the factors to be displayed
r = 1;
subplot(3,1,1)
plot(A(:,r),'k'),hold, plot(A_anls(:,r),'bd'), legend(...
    'Ground truth column','Estimated Column'), ylabel('A')
title(['Column ',num2str(r)])
subplot(3,1,2)
plot(B(:,r),'k'),hold, plot(B_anls(:,r),'bd'), ylabel('B')
subplot(3,1,3)
plot(C(:,r),'k'),hold, plot(C_anls(:,r),'bd'), ylabel('C')
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%

